// Copyright (C) 1999 Jean-Marc Valin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include "Node.h"
#include "ObjectRef.h"
#include "Buffer.h"
#include "Vector.h"

class PitchDecision;
DECLARE_NODE(PitchDecision)
/*Node

 * @name PitchDecision
 * @category Logic
 * @description No description available

 * @input_name INPUT
 * @input_description No description available

 * @output_name OUTPUT
 * @output_description No description available

 * @parameter_name TOTRUE
 * @parameter_description No description available

 * @parameter_name TOFALSE
 * @parameter_description No description available

 * @parameter_name THRESHOLD
 * @parameter_description No description available

 * @parameter_name INITIAL
 * @parameter_description No description available

END*/


class PitchDecision : public Node {
protected:
   /**The ID of the 'output' output*/
   int outputID;

   /**The ID of the 'input' input*/
   int inputID;

   /**The true ObjectRef used to return true*/
   ObjectRef trueObject;

   /**The false ObjectRef used to return true*/
   ObjectRef falseObject;

   /**True transition*/
   int trueTransition;
   
   /**False transition*/
   int falseTransition;

   /**PitchDecision threshold*/
   float threshold;

   /**current state (true/false)*/
   bool state;
   
   /**When this is high enough, the decision changes*/
   int decisionCount;
public:
   ///Constructor, takes the name of the node and a set of parameters
   PitchDecision(string nodeName, ParameterSet params) : Node (nodeName,params)
   {
      inputID = addInput ("INPUT");
      outputID = addOutput ("OUTPUT");
      trueObject = ObjectRef (new Bool(true));
      falseObject = ObjectRef (new Bool(false));
      
      trueTransition = dereference_cast<int> (parameters.get("TOTRUE"));
      falseTransition = dereference_cast<int> (parameters.get("TOFALSE"));
      threshold = dereference_cast<float> (parameters.get("THRESHOLD"));
      
      if (parameters.exist("INITIAL"))
         state = dereference_cast<bool> (parameters.get("INITIAL"));
      else 
         state = false;
      decisionCount=0;
   }
   
   virtual void specificInitialize()
   {
      this->Node::specificInitialize();
   }

   virtual void reset()
   {
      this->Node::reset();
      if (parameters.exist("INITIAL"))
         state = dereference_cast<bool> (parameters.get("INITIAL"));
      else 
         state = false;
      decisionCount=0;      
   }

   virtual ObjectRef getOutput(int output_id, int count);
};


ObjectRef PitchDecision::getOutput(int output_id, int count)
{
   
   for (int i=processCount+1;i<=count;i++)
   {
      NodeInput input = inputs[inputID];
      
      
      ObjectRef inputValue = input.node->getOutput(input.outputID,i);
      if (!inputValue->status)
      {
         //cerr << (object_cast<Vector<float> > (inputValue))[0] << endl;
         bool higher = (object_cast<Vector<float> > (inputValue))[0] >= threshold;
         if (state == true)
         {
            if (!higher)
            {
               decisionCount++;
               if (decisionCount >= falseTransition)
               {
                  state = false;
                  decisionCount=0;
               }
            }
            
         } else {
            if (higher)
            {
               decisionCount++;
               if (decisionCount >= trueTransition)
               {
                  state = true;
                  decisionCount=0;
               }
            }
            
         }
         
      }
      processCount=i;
   }
   if (state == true)
   {
      //if (!inputValue->status)
      //cout << 1 << "\n";
      return trueObject;
   }
   else
   {
      //if (!inputValue->status)
      //cout << 0 << "\n";
      return falseObject;
   }
}
